import greenfoot.*;

public class Player extends Actor
{
    private int direction = 1; // 1 = right and -1 = left
    private int speed = 4;
    private int vSpeed = 0;
    private int acceleration = 1;
    private int jumpStrength = 16;
    private boolean jumping;

    private int shootingCounter = 20; // Delay shooting

    private GreenfootImage run1r = new GreenfootImage("run1r.png");
    private GreenfootImage run2r = new GreenfootImage("run2r.png");
    private GreenfootImage run3r = new GreenfootImage("run3r.png");
    private GreenfootImage run4r = new GreenfootImage("run4r.png");
    
    private GreenfootImage run1l = new GreenfootImage("run1l.png");
    private GreenfootImage run2l = new GreenfootImage("run2l.png");
    private GreenfootImage run3l = new GreenfootImage("run3l.png");
    private GreenfootImage run4l = new GreenfootImage("run4l.png");
    private int frame = 1;
    private int animationCounter = 0;

    public void act() 
    {
        checkFall();
        checkKey();
        platformAbove();
        checkRightWalls();
        checkLeftWalls();
        shooting(); 
        if (shootingCounter > 0) shootingCounter--;
        animationCounter = (animationCounter+1)%4;
    }   

    public void checkKey()
    {

        if (Greenfoot.isKeyDown("right")) moveRight();
        if (Greenfoot.isKeyDown("left")) moveLeft();
        if (Greenfoot.isKeyDown("up") && jumping == false) jump();
    }

    public boolean shooting()
    {
        if (!Greenfoot.isKeyDown("space") || shootingCounter > 0) return false;
        if (direction == 1) getWorld().addObject(new ShootRight(), getX(), getY());
        else getWorld().addObject(new ShootLeft(), getX(), getY());
        shootingCounter = 20;
        return true;
    }

    public void moveRight()
    {
        direction = 1;
        setLocation(getX()+speed, getY());
        if (animationCounter == 0) animateRight();
    }

    public void animateRight()
    {
        frame = (frame+1)%4;
        if(frame == 0) setImage(run1r);
        if(frame == 1) setImage(run2r);
        if(frame == 2) setImage(run3r);
        if(frame == 3) setImage(run4r);
    }

    public void moveLeft()
    {
        direction = -1;
        setLocation(getX()-speed, getY());
        if(animationCounter == 0) animateLeft();
    }

    public void animateLeft()
    {
        frame = (frame+1)%4;
        if(frame == 0) setImage(run1l);
        if(frame == 1) setImage(run2l);
        if(frame == 2) setImage(run3l);
        if(frame == 3) setImage(run4l);
    }

    public boolean platformAbove()
    {
        int spriteHeight = getImage().getHeight();
        int yDistance = (int)(spriteHeight/-2);
        Actor ceiling = getOneObjectAtOffset(0, yDistance, Platform.class);
        if (ceiling == null) return false;
        vSpeed = 1;
        bopHead(ceiling);
        return true;
    }

    public boolean checkRightWalls()
    {
        int spriteWidth = getImage().getWidth();
        int xDistance = (int)(spriteWidth/2);
        Actor rightWall = getOneObjectAtOffset(xDistance, 0, Platform.class);
        if(rightWall == null) return false;
        stopByRightWall(rightWall);
        return true;
    }

    public void stopByRightWall(Actor rightWall)
    {
        int wallWidth = rightWall.getImage().getWidth();
        int newX = rightWall.getX()-(wallWidth + getImage().getWidth())/2;
        setLocation(newX-5, getY());

    }

    public boolean checkLeftWalls()
    {
        int spriteWidth = getImage().getWidth();
        int xDistance = (int)(spriteWidth/-2);
        Actor leftWall = getOneObjectAtOffset(xDistance, 0, Platform.class);
        if(leftWall == null) return false;
        stopByLeftWall(leftWall);
        return true;
    }

    public void stopByLeftWall(Actor leftWall)
    {
        int wallWidth = leftWall.getImage().getWidth();
        int newX = leftWall.getX()+(wallWidth+getImage().getWidth())/2;
        setLocation(newX+5, getY());

    }

    public void bopHead(Actor ceiling)
    {
        int ceilingHeight = ceiling.getImage().getHeight();
        int newY = ceiling.getY()+(ceilingHeight+getImage().getHeight())/2;
        setLocation(getX(), newY);
    }

    public void fall()
    {
        setLocation(getX(), getY()+vSpeed);
        if (vSpeed <= 9) vSpeed = vSpeed+acceleration;
        jumping = true;
    }

    public boolean onGround()
    {
        int spriteHeight = getImage().getHeight();
        int yDistance = (int)(spriteHeight/2)+5;
        Actor ground = getOneObjectAtOffset(0, getImage().getHeight()/2, Platform.class);
        if(ground == null)
        {
            jumping = true;
            return false;
        }
        else
        {
            moveToGround(ground);
            return true;
        }
    }

    public void moveToGround(Actor ground)
    {
        int groundHeight = ground.getImage().getHeight();
        int newY = ground.getY()-(groundHeight+getImage().getHeight())/2;
        setLocation(getX(), newY);
        jumping = false;
    }

    public void checkFall()
    {
        if (onGround()) vSpeed = 0; else fall();
    }

    public void jump()
    {
        vSpeed = vSpeed-jumpStrength;
        jumping = true;
        fall();
    }
}